/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet 
 */

package org.objectweb.dream.aggregator;

import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManagerType;
import org.objectweb.fractal.fraclet.annotation.annotations.Attribute;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * This aggregator aggregates a fixed number of messages. These messages are
 * collected on its Pull input. The number of messages to be aggregated is set
 * using the <code>PullPullAggregatorFixedNumberAttributeController</code>
 * interface generated by fraclet.
 */
@DreamComponent(controllerDesc = "dreamPrimitive")
@Provides(interfaces = @Interface(name = "out-pull", signature = Pull.class))
public class PullPullAggregatorFixedNumber
    implements
      Pull
{
  @Requires(name = "in-pull")
  private Pull                 inPullItf;

  /** the message manager client interface of this component */
  @Requires(name = "message-manager")
  protected MessageManagerType messageManagerItf;

  /**
   * the number of messages that must be aggregated
   */
  @Attribute(argument = "nbMessageToAggregate")
  private int                  nbMessageToAggregate;

  // ---------------------------------------------------------------------------
  // Implementation of the Pull interface.
  // ---------------------------------------------------------------------------
  /**
   * @see Pull#pull()
   */
  public Message pull() throws PullException
  {
    Message outMessage = messageManagerItf.createMessage();
    for (int i = 0; i < nbMessageToAggregate; i++)
    {
      messageManagerItf.addSubMessage(outMessage, inPullItf.pull());
    }
    return outMessage;
  }
}