/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;

/**
 * Interface used by ChannelOut to resolve the destination of messages.
 */
@Interface(name = "destination-resolver")
public interface DestinationResolver
{
  /** Default name for interfaces of this type. */
  String ITF_NAME = "destination-resolver";

  /**
   * Return the {@link ExportIdentifier} which identifies the destination of the
   * given message.
   * 
   * @param message a message.
   * @return the destination identifier of the message.
   * @throws PushException if something goes wrong.
   */
  ExportIdentifier resolveDestination(Message message) throws PushException;
}
