/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.channel;

import java.net.ServerSocket;

import org.objectweb.dream.protocol.IPExportIdentifier;
import org.objectweb.dream.protocol.InvalidExportIdentifierException;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;

/**
 * Interface provided by server session factory. A server session factory manages
 * a set of {@link ServerSocket}. When a connection is opened on one of this
 * server socket, a new server session is created and passed to the protocol
 * component through the {@link ConnectionManager} interface.
 */
@Interface(name = ConnectionFactoryItf.ITF_NAME)
public interface ConnectionFactoryItf
{

  /** The commonly used name of this interface. */
  String ITF_NAME = "connection-factory";

  /**
   * Adds a new server socket.
   * 
   * @param serverSocket the server socket.
   * @param identifier the export identifier associated with the server socket.
   */
  void addServerSocket(ServerSocket serverSocket, IPExportIdentifier identifier);

  /**
   * Removes and closes the server socket associated with the given identifier.
   * 
   * @param identifier an identifier of a previously added server socket.
   * @throws InvalidExportIdentifierException if the given export identifier is
   *           unknown.
   */
  void removeServerSocket(IPExportIdentifier identifier)
      throws InvalidExportIdentifierException;
}
