/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.pushwithreturn;


import org.objectweb.dream.message.ChunkFactoryReference;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManagerType;
import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * This generator generates <code>KeyInteger</code> objects with an increasing
 * integer. It adds a <code>KeyChunk</code> chunk to the message.
 */
@DreamComponent(controllerDesc = "dreamPrimitive")
public class KeyGeneratorInteger 
    implements
      KeyGeneratorItf
{
  /** The "All" Integer key. */
  public static final KeyInteger          ALL_INTEGER_KEY = new KeyInteger(0,
                                                              true);

  /** The integer contained in the last generated key. */
  private int                             lastInSequence  = 0;
  private ChunkFactoryReference<KeyChunk> keyChunkFactory;

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  @Requires(name = "message-manager")
  private MessageManagerType                  messageManagerItf;

  // ---------------------------------------------------------------------------
  // Implementation of the KeyGeneratorItf interface.
  // ---------------------------------------------------------------------------
  /**
   * @see KeyGeneratorItf#generateKey(Message)
   */
  public Key generateKey(Message message) throws Exception
  {
    KeyChunk keyChunk = (KeyChunk) messageManagerItf.getChunk(message,
        KeyChunk.DEFAULT_NAME);
    if (keyChunk != null)
    {
      return keyChunk.getKey();
    }
    KeyInteger key = new KeyInteger(++lastInSequence, false);
    if (keyChunkFactory == null)
    {
      keyChunkFactory = messageManagerItf.getChunkFactory(KeyChunk.class);
    }
    keyChunk = messageManagerItf.createChunk(keyChunkFactory);
    keyChunk.setKey(key);
    messageManagerItf.addChunk(message, KeyChunk.DEFAULT_NAME, keyChunk);
    return key;
  }
}
