/**
 * DREAM
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.queue;

import org.objectweb.fractal.fraclet.annotation.annotations.Interface;

/**
 * Interface that can be used by a queue sorting messages according to a
 * sequence number (i.e. a long) to indicate that sequence numbers are missing.
 */
@Interface(name = MissingSequenceNumber.ITF_NAME)
public interface MissingSequenceNumber
{
  /**
   * The commonly used name to refer to this interface.
   */
  String ITF_NAME = "missing-sequence-number";

  /**
   * This method is called to specify that messages ranging from
   * <code>snFrom</code> to <code>snTo</code> are missing.
   * 
   * @param snFrom the first sequence number in sequence that is missing.
   * @param snTo the last sequence number in sequence that is missing.
   */
  void missingSequenceNumber(long snFrom, long snTo);

  /**
   * This method is called to specify that a sequence number is no longer
   * missing.
   * 
   * @param sn the sequence number that is no longer missing.
   */
  void noLongerMissing(long sn);
}