/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Pierre Garcia
 * Contributor(s):
 */

package org.objectweb.dream.queue.handler.outgoing;


import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.queue.buffer.BufferRemove;
import org.objectweb.dream.dreamannotation.DreamComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;

/**
 * Abstract implementation of an outgoing handler. It implements the
 * {@link Pull} interface.
 * <p>
 * An outgoing handler aims at treating outgoing messages coming from a buffer
 * <p>
 * Outgoing handler developers can inherit this class. They must implement one
 * method:
 * <ul>
 * <li><code>pull(message)</code> pull a message from the related buffer.
 * </li>
 * </p>
 */
@DreamComponent()
@Provides(interfaces = @Interface(name = "out-pull", signature = Pull.class))
public abstract class AbstractOutgoingHandler 
    implements
      Pull
{

  // ------------------------------------------------------------------------
  // Client interfaces.
  // ------------------------------------------------------------------------

  /**
   * The buffer from which pulled messages are removed.
   */
  @Requires(name = BufferRemove.ITF_NAME)
  protected BufferRemove bufferRemove;

  // ------------------------------------------------------------------------
  // Method that should be overridden by concrete sub classes.
  // ------------------------------------------------------------------------

  /**
   * @see Pull#pull()
   */
  public abstract Message pull() throws PullException;

}
