/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.queue.keyed.handler.incoming;

import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.dreamannotation.DreamComponent;

/**
 * A, incoming handler that implements the
 * {@link org.objectweb.dream.queue.keyed.KeyedPush}interface. It adds
 * {@link org.objectweb.dream.queue.keyed.KeyedPush#push(Message, Object) pushed}
 * messages into a {@link org.objectweb.dream.queue.buffer.BufferAdd buffer}bound
 * to it. This component uses
 * {@link org.objectweb.dream.queue.buffer.BufferAdd#add(Message) blocking calls for adding}
 * messages into the buffer.
 */
@DreamComponent(controllerDesc = "dreamUnstoppablePrimitive")
public class KeyedIncomingHandlerBlocking extends AbstractKeyedIncomingHandler
{

  // ---------------------------------------------------------------------------
  // Implementation of the KeyedPush interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.queue.keyed.KeyedPush#push(Message, Object)
   */
  public void push(Message message, Object key) throws PushException
  {
    try
    {
      keyedBufferAddItf.add(message, key);
    }
    catch (InterruptedException e)
    {
      throw new PushException(
          "interrupted while adding message into the buffer", e);
    }
  }
}