/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Romain Lenglet
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.queue.keyed.handler.incoming;

import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.queue.buffer.BufferOverflowException;
import org.objectweb.dream.dreamannotation.DreamComponent;

/**
 * A push incoming handler that implements the
 * {@link org.objectweb.dream.queue.keyed.KeyedPush}interface. It adds
 * {@link org.objectweb.dream.queue.keyed..KeyedPush#push(Message, Object) pushed}messages
 * into a {@link org.objectweb.dream.queue.keyed.buffer.KeyedBufferAdd buffer}bound
 * to it. This component uses
 * {@link org.objectweb.dream.queue.keyed.buffer.KeyedBufferAdd#tryAdd(Message, Object) non blocking calls for trying to add}
 * messages into the buffer: if a message cannot be added, an
 * {@link PushException exception}is thrown. In any case the message is never
 * deleted by this component.
 */
@DreamComponent(controllerDesc = "dreamUnstoppablePrimitive")
public class KeyedIncomingHandlerExceptionThrowing
    extends
      AbstractKeyedIncomingHandler
{

  // ---------------------------------------------------------------------------
  // Implementation of the KeyedPush interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.queue.keyed.KeyedPush#push(Message, Object)
   */
  public void push(Message message, Object key) throws PushException
  {
    if (!keyedBufferAddItf.tryAdd(message, key))
    {
      throw new BufferOverflowException(
          "overflow in buffer; cannot add pushed message");
    }
  }
}