/**
 * Dream
 * Copyright (C) 2003 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: vivien.quema@inrialpes.fr
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.router;

import java.util.Collection;
import java.util.Iterator;

import org.objectweb.dream.Push;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.dreamannotation.DreamComponent;

/**
 * This router implements a round-robin choice of its outputs. Note that the
 * <code>defaultOutPushItf</code> is only chosen if no route is bound to the
 * router.
 */
@DreamComponent(controllerDesc = "dreamPrimitive")
public class RoundRobinRouter extends AbstractRouterImpl
{

  private int nextOutPushIndex = -1;
  Push[]      itfs;

  /**
   * @see org.objectweb.dream.router.AbstractRouterImpl#getOutput(Message)
   */
  protected Push getOutput(Message message)
  {
    if (!initialized)
    {
      Collection values = outPushMap.values();
      Iterator iter = values.iterator();
      itfs = new Push[values.size()];
      int i = 0;
      while (iter.hasNext())
      {
        itfs[i] = (Push) iter.next();
        i++;
      }
      initialized = true;
    }
    if (itfs.length > 0)
    {
      nextOutPushIndex = (nextOutPushIndex + 1) % itfs.length;
      return itfs[nextOutPushIndex];
    }
    return null;
  }

}