/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.synchro;

import org.objectweb.dream.dreamannotation.DreamComponent;

/**
 * FIFO implementation of the {@link SemaphoreItf}interface. Waiting requests will
 * be satisfied in FIFO order.
 * <p>
 * <b>Note: Inspired by Doug Lea's implementation. </b>
 */
@DreamComponent(controllerDesc = "dreamUnstoppablePrimitive")
public class SemaphoreFIFO extends AbstractSemaphoreQueuedImpl
{

  // ---------------------------------------------------------------------------
  // Implementation of the SemaphoreItf interface.
  // ---------------------------------------------------------------------------

  /**
   * Constructor.
   */
  public SemaphoreFIFO()
  {
    super(new FIFOWaitQueue());
  }

  // ---------------------------------------------------------------------------
  // Inner class.
  // ---------------------------------------------------------------------------

  /**
   * Simple linked list queue used in FIFOSemaphore. Methods are not
   * synchronized; they depend on synch of callers
   */
  protected static class FIFOWaitQueue extends WaitQueue
  {
    protected WaitNode head = null;
    protected WaitNode tail = null;

    protected void insert(WaitNode w)
    {
      if (tail == null)
      {
        head = w;
        tail = w;
      }
      else
      {
        tail.next = w;
        tail = w;
      }
    }

    protected WaitNode extract()
    {
      if (head == null) return null;
      WaitNode w = head;
      head = w.next;
      if (head == null)
      {
        tail = null;
      }
      w.next = null;
      return w;
    }
  }

}
