/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.time;


import org.objectweb.dream.util.EmptyStringArray;
import org.objectweb.dream.dreamannotation.DreamComponent;

/**
 * Basic implementation of a time stamp component. Such component implements the
 * <code>SetTimeStamp</code> and <code>GetTimeStamp</code> interfaces.
 */
@DreamComponent(controllerDesc = "dreamPrimitive")
public class TimeStamp 
    implements
      GetTimeStamp,
      SetTimeStamp
{

  /** The time stamp. */
  private long timeStamp = 0;

  // -------------------------------------------------------------------------
  // Implementation of the SetTimeStamp interface
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.time.SetTimeStamp#setTimeStamp()
   */
  public long setTimeStamp()
  {
    this.timeStamp = System.currentTimeMillis();
    return timeStamp;
  }

  // -------------------------------------------------------------------------
  // Implementation of the GetTimeStamp interface
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.time.GetTimeStamp#getTimeStamp()
   */
  public long getTimeStamp()
  {
    return timeStamp;
  }

  // -------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public synchronized String[] listFc()
  {
    return EmptyStringArray.EMPTY_STRING_ARRAY;
  }

}